/*
 * CSS formatting for IGWN Robot Auth with Furo
 *
 * Copyright (c) 2025 Cardiff University
 */

/* remove block spacing for multi line parameter lists */
dl.py > dt.sig.sig-object.py > dl {
  margin-block-start: 0px;
  margin-block-end: 0px;
}

/* outdent trailing parenthesis for multi line parameter lists */
dl.py > dt.sig.sig-object.py > dl ~ span.sig-paren {
  margin-left: -2.5em;
}
