# Copyright (c) 2025 Cardiff University
# SPDX-License-Identifier: MIT

"""Manage authorisation credentials for IGWN Robot identities."""

__author__ = "Duncan Macleod <duncan.macleod@ligo.org>"

from .tools.get import main as get_main

if __name__ == "__main__":
    get_main()
